﻿using System.Drawing;
using System.Drawing.Drawing2D;

namespace TestProject
{
    /// <summary>
    /// 그래픽스 헬퍼
    /// </summary>
    public class GraphicsHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 그래픽스
        /// </summary>
        private Graphics graphics;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - GraphicsHelper(graphics)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        public GraphicsHelper(Graphics graphics)
        {
            this.graphics = graphics;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 선 그리기 - DrawLine(pen, startPoint, endPoint)

        /// <summary>
        /// 선 그리기
        /// </summary>
        /// <param name="pen">펜</param>
        /// <param name="startPoint">시작 포인트</param>
        /// <param name="endPoint">종료 포인트</param>
        public void DrawLine(Pen pen, Point startPoint, Point endPoint)
        {
            Point point1 = new Point(startPoint.X, startPoint.Y);
            Point point2 = new Point(endPoint.X  , endPoint.Y  );

            if(point2.X < point1.X)
            {
                Point temporaryPoint = point2;

                point2 = point1;

                point1 = temporaryPoint;
            }

            if(point1.Y == point2.Y)
            {
                point2.X--;
            } 
            else if(point1.X == point2.X)
            {
                point2.Y--;
            }
            else if(point2.Y > point1.Y)
            {
                point2.X--;
                point2.Y--;
            }
            else if(point2.Y < point1.Y)
            {
                point1.Y--;
                point2.X--;
            }

            graphics.DrawLine(pen, point1, point2);
        }

        #endregion
        #region 선 그리기 - DrawLine(pen, startX, startY, endX, endY)

        /// <summary>
        /// 선 그리기
        /// </summary>
        /// <param name="pen">펜</param>
        /// <param name="startX">시작 X</param>
        /// <param name="startY">시작 Y</param>
        /// <param name="endX">종료 X</param>
        /// <param name="endY">종료 Y</param>
        public void DrawLine(Pen pen, int startX, int startY, int endX, int endY)
        {
            DrawLine(pen, new Point(startX, startY), new Point(endX, endY));
        }

        #endregion

        #region 사각형 그리기 - DrawRectangle(pen, rectangle)

        /// <summary>
        /// 사각형 그리기
        /// </summary>
        /// <param name="pen">펜</param>
        /// <param name="rectangle">사각형</param>
        public void DrawRectangle(Pen pen, Rectangle rectangle)
        {
            if(pen.Width == 1) 
            {
                Rectangle rectangle2 = rectangle;

                rectangle2.Width--;
                rectangle2.Height--;

                graphics.DrawRectangle(pen, rectangle2);
            } 
            else 
            {
                PenAlignment penAlignment = pen.Alignment;

                pen.Alignment = PenAlignment.Inset;

                graphics.DrawRectangle(pen, rectangle);

                pen.Alignment = penAlignment;
            }
        }

        #endregion
        #region 사각형 그리기 - DrawRectangle(pen, x, y, width, height)

        /// <summary>
        /// 사각형 그리기
        /// </summary>
        /// <param name="pen">펜</param>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        public void DrawRectangle(Pen pen, int x, int y, int width, int height)
        {
            Rectangle rectangle = new Rectangle(x, y, width, height);

            DrawRectangle(pen, rectangle);
        }

        #endregion

        #region 사각형 채우기 - FillRectangle(brush, rectangle)

        /// <summary>
        /// 사각형 채우기
        /// </summary>
        /// <param name="brush">브러시</param>
        /// <param name="rectangle">사각형</param>
        public void FillRectangle(Brush brush, Rectangle rectangle)
        {
            graphics.FillRectangle(brush, rectangle);
        }

        #endregion

        #region 3D 선 그리기 - Draw3DLine(startPoint, endPoint, style, depth)

        /// <summary>
        /// 3D 선 그리기
        /// </summary>
        /// <param name="startPoint">시작 포인트</param>
        /// <param name="endPoint">종료 포인트</param>
        /// <param name="style">스타일</param>
        /// <param name="depth">깊이</param>
        public void Draw3DLine(Point startPoint, Point endPoint, Style3D style, int depth)
        {
            if(startPoint.X != endPoint.X && startPoint.Y != endPoint.Y)
            {
                throw new InvalidArgumentException();
            }

            if(depth != 1 && depth != 2)
            {
                throw new InvalidArgumentException();
            }

            if(depth == 1) 
            {
                switch(style)
                {
                    case Style3D.Inset  :
                    case Style3D.Groove :

                        if(startPoint.Y == endPoint.Y)
                        {
                            DrawLine(SystemPens.ControlDark, startPoint, endPoint);

                            Point point1 = new Point(startPoint.X, startPoint.Y + 1);
                            Point point2 = new Point(endPoint.X  , endPoint.Y   + 1);

                            DrawLine(SystemPens.ControlLightLight, point1, point2);
                        } 
                        else 
                        {
                            DrawLine(SystemPens.ControlDark, startPoint, endPoint);

                            Point point1 = new Point(startPoint.X + 1, startPoint.Y);
                            Point point2 = new Point(endPoint.X + 1  , endPoint.Y  );

                            DrawLine(SystemPens.ControlLightLight, point1, point2);
                        }

                        break;

                    case Style3D.Raised :
                    case Style3D.Ridge  :

                        if(startPoint.Y == endPoint.Y)
                        {
                            DrawLine(SystemPens.ControlLightLight, startPoint, endPoint);

                            Point point1 = new Point(startPoint.X, startPoint.Y + 1);
                            Point point2 = new Point(endPoint.X  , endPoint.Y   + 1);

                            DrawLine(SystemPens.ControlDark, point1, point2);
                        }
                        else
                        {
                            DrawLine(SystemPens.ControlLightLight, startPoint, endPoint);

                            Point point1 = new Point(startPoint.X + 1, startPoint.Y);
                            Point point2 = new Point(endPoint.X   + 1, endPoint.Y  );

                            DrawLine(SystemPens.ControlDark, point1, point2);
                        }

                        break;
                }
            }
            else if(depth == 2)
            {
                switch(style)
                {
                    case Style3D.Inset  :
                    case Style3D.Groove :

                        if(startPoint.Y == endPoint.Y)
                        {
                            DrawLine(SystemPens.ControlDarkDark, startPoint, endPoint);

                            Point point1 = new Point(startPoint.X, startPoint.Y + 1);
                            Point point2 = new Point(endPoint.X  , endPoint.Y   + 1);

                            DrawLine(SystemPens.ControlDark, point1, point2);

                            point1.Y++;
                            point2.Y++;

                            DrawLine(SystemPens.ControlLight, point1, point2);

                            point1.Y++;
                            point2.Y++;

                            DrawLine(SystemPens.ControlLightLight, point1, point2);
                        } 
                        else 
                        {
                            DrawLine(SystemPens.ControlDarkDark, startPoint, endPoint);

                            Point point1 = new Point(startPoint.X + 1, startPoint.Y);
                            Point point2 = new Point(endPoint.X   + 1, endPoint.Y  );

                            DrawLine(SystemPens.ControlDark, point1, point2);

                            point1.X++;
                            point2.X++;

                            DrawLine(SystemPens.ControlLight, point1, point2);

                            point1.X++;
                            point2.X++;

                            DrawLine(SystemPens.ControlLightLight, point1, point2);
                        }

                        break;

                    case Style3D.Raised :
                    case Style3D.Ridge  :

                        if(startPoint.Y == endPoint.Y)
                        {
                            DrawLine(SystemPens.ControlLightLight, startPoint, endPoint);

                            Point point1 = new Point(startPoint.X, startPoint.Y + 1);
                            Point point2 = new Point(endPoint.X  , endPoint.Y   + 1);

                            DrawLine(SystemPens.ControlLight, point1, point2);

                            point1.Y++;
                            point2.Y++;

                            DrawLine(SystemPens.ControlDark, point1, point2);

                            point1.Y++;
                            point2.Y++;

                            DrawLine(SystemPens.ControlDarkDark, point1, point2);
                        }
                        else
                        {
                            DrawLine(SystemPens.ControlLightLight, startPoint, endPoint);

                            Point point1 = new Point(startPoint.X + 1, startPoint.Y);
                            Point point2 = new Point(endPoint.X   + 1, endPoint.Y  );

                            DrawLine(SystemPens.ControlLight, point1, point2);

                            point1.X++;
                            point2.X++;

                            DrawLine(SystemPens.ControlDark, point1, point2);

                            point1.X++;
                            point2.X++;

                            DrawLine(SystemPens.ControlDarkDark, point1, point2);
                        }

                        break;
                }
            }
        }

        #endregion
        #region 3D 선 그리기 - Draw3DLine(startX, startY, endX, endY, style, depth)

        /// <summary>
        /// 3D 선 그리기
        /// </summary>
        /// <param name="startX">시작 X</param>
        /// <param name="startY">시작 Y</param>
        /// <param name="endX">종료 X</param>
        /// <param name="endY">종료 Y</param>
        /// <param name="style">스타일</param>
        /// <param name="depth">깊이</param>
        public void Draw3DLine(int startX, int startY, int endX, int endY, Style3D style, int depth)
        {
            Draw3DLine(new Point(startX, startY), new Point(endX, endY), style, depth);
        }

        #endregion

        #region 3D 사각형 그리기 - Draw3DRectangle(rectangle, style, depth)

        /// <summary>
        /// 3D 사각형 그리기
        /// </summary>
        /// <param name="rectangle">사각형</param>
        /// <param name="style">스타일</param>
        /// <param name="depth">깊이</param>
        public void Draw3DRectangle(Rectangle rectangle, Style3D style, int depth)
        {
            if(depth != 1 && depth != 2)
            {
                throw new InvalidArgumentException();
            }

            if(depth == 1) 
            {
                switch(style)
                {
                    case Style3D.Ridge :

                        DrawDepth1RidgeGrooveRectangle(rectangle, SystemPens.ControlLightLight, SystemPens.ControlDark);

                        break;

                    case Style3D.Groove :

                        DrawDepth1RidgeGrooveRectangle(rectangle, SystemPens.ControlDark, SystemPens.ControlLightLight);

                        break;

                    case Style3D.Raised :

                        DrawDepth1InsetRaisedRectangle(rectangle, SystemPens.ControlLightLight, SystemPens.ControlDark);

                        Rectangle rectangle2 = new Rectangle(rectangle.Location, rectangle.Size);

                        rectangle2.Inflate(-1, -1);

                        FillRectangle(SystemBrushes.Control, rectangle2);

                        break;

                    case Style3D.Inset :

                        DrawDepth1InsetRaisedRectangle(rectangle, SystemPens.ControlDark, SystemPens.ControlLightLight);

                        Rectangle rectangle3 = new Rectangle(rectangle.Location, rectangle.Size);

                        rectangle3.Inflate(-1, -1);

                        FillRectangle(SystemBrushes.Control, rectangle3);

                        break;
                }
            }
            else if(depth == 2)
            {
                switch(style)
                {
                    case Style3D.Ridge :

                        DrawDepth2RidgeGrooveRectangle
                        (
                            rectangle,
                            SystemPens.ControlLightLight,
                            SystemPens.ControlLight,
                            SystemPens.ControlDark,
                            SystemPens.ControlDarkDark
                        );

                        break;

                    case Style3D.Groove :

                        DrawDepth2RidgeGrooveRectangle
                        (
                            rectangle,
                            SystemPens.ControlDarkDark,
                            SystemPens.ControlDark,
                            SystemPens.ControlLight,
                            SystemPens.ControlLightLight
                        );

                        break;

                    case Style3D.Raised :

                        DrawDepth2InsetRaisedRectangle
                        (
                            rectangle,
                            SystemPens.ControlLightLight,
                            SystemPens.ControlLight,
                            SystemPens.ControlDark,
                            SystemPens.ControlDarkDark
                        );

                        Rectangle rectangle2 = new Rectangle(rectangle.Location, rectangle.Size);

                        rectangle2.Inflate(-2, -2);

                        FillRectangle(SystemBrushes.Control, rectangle2);

                        break;

                    case Style3D.Inset :

                        DrawDepth2InsetRaisedRectangle
                        (
                            rectangle,
                            SystemPens.ControlDarkDark,
                            SystemPens.ControlDark,
                            SystemPens.ControlLight,
                            SystemPens.ControlLightLight
                        );

                        Rectangle rectangle3 = new Rectangle(rectangle.Location, rectangle.Size);

                        rectangle3.Inflate(-2, -2);

                        FillRectangle(SystemBrushes.Control, rectangle3);

                        break;
                }
            }
        }

        #endregion
        #region 3D 사각형 그리기 - Draw3DRectangle(x, y, width, height, style, depth)

        /// <summary>
        /// 3D 사각형 그리기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="style">스타일</param>
        /// <param name="depth">깊이</param>
        public void Draw3DRectangle(int x, int y, int width, int height, Style3D style, int depth)
        {
            Draw3DRectangle(new Rectangle(x, y, width, height), style, depth);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 깊이 1 INSET/RAISED 사각형 그리기 - DrawDepth1InsetRaisedRectangle(rectangle, pen1, pen2)

        /// <summary>
        /// 깊이 1 INSET/RAISED 사각형 그리기
        /// </summary>
        /// <param name="rectangle">사각형</param>
        /// <param name="pen1">펜 1</param>
        /// <param name="pen2">펜 2</param>
        private void DrawDepth1InsetRaisedRectangle(Rectangle rectangle, Pen pen1, Pen pen2)
        {
            Point point1 = new Point(rectangle.Left                  , rectangle.Top);
            Point point2 = new Point(rectangle.Left + rectangle.Width, rectangle.Top);

            DrawLine(pen1, point1, point2);

            point1 = new Point(rectangle.Left, rectangle.Top                   );
            point2 = new Point(rectangle.Left, rectangle.Top + rectangle.Height);

            DrawLine(pen1, point1, point2);

            point1 = new Point(rectangle.Left                  , rectangle.Top + rectangle.Height - 1);
            point2 = new Point(rectangle.Left + rectangle.Width, rectangle.Top + rectangle.Height - 1);

            DrawLine(pen2, point1, point2);

            point1 = new Point(rectangle.Left + rectangle.Width - 1, rectangle.Top                   );
            point2 = new Point(rectangle.Left + rectangle.Width - 1, rectangle.Top + rectangle.Height);

            DrawLine(pen2, point1, point2);
        }

        #endregion
        #region 깊이 1 RIDGE/GROOVE 사각형 그리기 - DrawDepth1RidgeGrooveRectangle(rectangle, pen1, pen2)

        /// <summary>
        /// 깊이 1 RIDGE/GROOVE 사각형 그리기
        /// </summary>
        /// <param name="rectangle">사각형</param>
        /// <param name="pen1">펜 1</param>
        /// <param name="pen2">펜 2</param>
        private void DrawDepth1RidgeGrooveRectangle(Rectangle rectangle, Pen pen1, Pen pen2)
        {
            Point point1 = new Point(rectangle.Left                  , rectangle.Top);
            Point point2 = new Point(rectangle.Left + rectangle.Width, rectangle.Top);

            DrawLine(pen1, point1, point2);

            point1.X++;
            point2.X--;

            point1.Y++;
            point2.Y++;

            DrawLine(pen2, point1, point2);

            point1 = new Point(rectangle.Left, rectangle.Top                   );
            point2 = new Point(rectangle.Left, rectangle.Top + rectangle.Height);

            DrawLine(pen1, point1, point2);

            point1.X++;
            point2.X++;

            point1.Y++;
            point2.Y--;

            DrawLine(pen2, point1, point2);

            point1 = new Point(rectangle.Left                  , rectangle.Top + rectangle.Height - 1);
            point2 = new Point(rectangle.Left + rectangle.Width, rectangle.Top + rectangle.Height - 1);

            DrawLine(pen2, point1, point2);

            point1.X++;
            point2.X--;

            point1.Y--;
            point2.Y--;

            DrawLine(pen1, point1, point2);

            point1 = new Point(rectangle.Left + rectangle.Width - 1, rectangle.Top                   );
            point2 = new Point(rectangle.Left + rectangle.Width - 1, rectangle.Top + rectangle.Height);

            DrawLine(pen2, point1, point2);

            point1.X--;
            point2.X--;

            point1.Y++;
            point2.Y--;

            DrawLine(pen1, point1, point2);
        }

        #endregion

        #region 깊이 2 INSERT/RAISED 사각형 그리기 - DrawDepth2InsetRaisedRectangle(rectangle, pen1, pen2, pen3, pen4)

        /// <summary>
        /// 깊이 2 INSERT/RAISED 사각형 그리기
        /// </summary>
        /// <param name="rectangle">사각형</param>
        /// <param name="pen1">펜 1</param>
        /// <param name="pen2">펜 2</param>
        /// <param name="pen3">펜 3</param>
        /// <param name="pen4">펜 4</param>
        private void DrawDepth2InsetRaisedRectangle(Rectangle rectangle, Pen pen1, Pen pen2, Pen pen3, Pen pen4)
        {
            Point point1 = new Point(rectangle.Left                  , rectangle.Top);
            Point point2 = new Point(rectangle.Left + rectangle.Width, rectangle.Top);

            DrawLine(pen1, point1, point2);

            point1.X++;
            point2.X--;

            point1.Y++;
            point2.Y++;

            DrawLine(pen2, point1, point2);

            point1 = new Point(rectangle.Left, rectangle.Top                   );
            point2 = new Point(rectangle.Left, rectangle.Top + rectangle.Height);

            DrawLine(pen1, point1, point2);

            point1.X++;
            point2.X++;

            point1.Y++;
            point2.Y--;

            DrawLine(pen2, point1, point2);

            point1 = new Point(rectangle.Left                  , rectangle.Top + rectangle.Height - 1);
            point2 = new Point(rectangle.Left + rectangle.Width, rectangle.Top + rectangle.Height - 1);

            DrawLine(pen4, point1, point2);

            point1.X++;
            point2.X--;

            point1.Y--;
            point2.Y--;

            DrawLine(pen3, point1, point2);

            point1 = new Point(rectangle.Left + rectangle.Width - 1, rectangle.Top                   );
            point2 = new Point(rectangle.Left + rectangle.Width - 1, rectangle.Top + rectangle.Height);

            DrawLine(pen4, point1, point2);

            point1.X--;
            point2.X--;

            point1.Y++;
            point2.Y--;

            DrawLine(pen3, point1, point2);
        }

        #endregion
        #region 깊이 2 RIDGE/GROOVE 사각형 그리기 - DrawDepth2RidgeGrooveRectangle(rectangle, pen1, pen2, pen3, pen4)

        /// <summary>
        /// 깊이 2 RIDGE/GROOVE 사각형 그리기
        /// </summary>
        /// <param name="rectangle">사각형</param>
        /// <param name="pen1">펜 1</param>
        /// <param name="pen2">펜 2</param>
        /// <param name="pen3">펜 3</param>
        /// <param name="pen4">펜 4</param>
        private void DrawDepth2RidgeGrooveRectangle(Rectangle rectangle, Pen pen1, Pen pen2, Pen pen3, Pen pen4)
        {
            Point point1 = new Point(rectangle.Left                  , rectangle.Top);
            Point point2 = new Point(rectangle.Left + rectangle.Width, rectangle.Top);

            DrawLine(pen1, point1, point2);

            point1.X++;
            point2.X--;

            point1.Y++;
            point2.Y++;

            DrawLine(pen2, point1, point2);

            point1.X++;
            point2.X--;

            point1.Y++;
            point2.Y++;

            DrawLine(pen3, point1, point2);

            point1.X++;
            point2.X--;

            point1.Y++;
            point2.Y++;

            DrawLine(pen4, point1, point2);

            point1 = new Point(rectangle.Left, rectangle.Top                   );
            point2 = new Point(rectangle.Left, rectangle.Top + rectangle.Height);

            DrawLine(pen1, point1, point2);

            point1.X++;
            point2.X++;

            point1.Y++;
            point2.Y--;

            DrawLine(pen2, point1, point2);

            point1.X++;
            point2.X++;

            point1.Y++;
            point2.Y--;

            DrawLine(pen3, point1, point2);

            point1.X++;
            point2.X++;

            point1.Y++;
            point2.Y--;

            DrawLine(pen4, point1, point2);

            point1 = new Point(rectangle.Left                  , rectangle.Top + rectangle.Height - 1);
            point2 = new Point(rectangle.Left + rectangle.Width, rectangle.Top + rectangle.Height - 1);

            DrawLine(pen4, point1, point2);

            point1.X++;
            point2.X--;

            point1.Y--;
            point2.Y--;

            DrawLine(pen3, point1, point2);

            point1.X++;
            point2.X--;

            point1.Y--;
            point2.Y--;

            DrawLine(pen2, point1, point2);

            point1.X++;
            point2.X--;

            point1.Y--;
            point2.Y--;

            DrawLine(pen1, point1, point2);

            point1 = new Point(rectangle.Left + rectangle.Width - 1, rectangle.Top                   );
            point2 = new Point(rectangle.Left + rectangle.Width - 1, rectangle.Top + rectangle.Height);

            DrawLine(pen4, point1, point2);

            point1.X--;
            point2.X--;

            point1.Y++;
            point2.Y--;

            DrawLine(pen3, point1, point2);

            point1.X--;
            point2.X--;

            point1.Y++;
            point2.Y--;

            DrawLine(pen2, point1, point2);

            point1.X--;
            point2.X--;

            point1.Y++;
            point2.Y--;

            DrawLine(pen1, point1, point2);
        }

        #endregion
    }
}