﻿using System.Drawing;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 캡처 변경시 윈도우
        /// </summary>
        private CaptureChangedWindow captureChangedWindow;

        /// <summary>
        /// 버튼 사각형
        /// </summary>
        private Rectangle buttonRectangle = new Rectangle(400, 100, 100, 40);

        /// <summary>
        /// 버튼 내에서 마우스 DOWN 여부
        /// </summary>
        private bool mouseDownInButton;

        /// <summary>
        /// 버튼 DOWN 여부
        /// </summary>
        private bool buttonDown;

        /// <summary>
        /// 적색 사각형
        /// </summary>
        private Rectangle redRectangle = new Rectangle(400, 200, 100, 40);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.captureChangedWindow = new CaptureChangedWindow();

            this.captureChangedWindow.AssignHandle(Handle);

            MouseDown                                += Form_MouseDown;
            MouseMove                                += Form_MouseMove;
            MouseUp                                  += Form_MouseUp;
            Paint                                    += Form_Paint;
            this.captureChangedWindow.CaptureChanged += captureChangedWindow_CaptureChanged;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 마우스 DOWN 처리하기 - Form_MouseDown(sender, e)

        /// <summary>
        /// 폼 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_MouseDown(object sender, MouseEventArgs e)
        {
            if(e.Button == MouseButtons.Left && this.buttonRectangle.Contains(e.X, e.Y))
            {
                this.mouseDownInButton = true;

                this.buttonDown = true;

                DrawButton(this.buttonRectangle, null, false);
            }
        }

        #endregion
        #region 폼 마우스 이동시 처리하기 - Form_MouseMove(sender, e)

        /// <summary>
        /// 폼 마우스 이동시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_MouseMove(object sender, MouseEventArgs e)
        {
            if(this.mouseDownInButton)
            {
                if(this.buttonRectangle.Contains(e.X, e.Y))
                {
                    if(!this.buttonDown)
                    {
                        DrawButton(buttonRectangle, null, false);

                        this.buttonDown = true;
                    }
                }
                else
                {
                    if(this.buttonDown)
                    {
                        DrawButton(this.buttonRectangle, null, true);

                        this.buttonDown = false;
                    }
                }
            }

            if(this.redRectangle.Contains(new Point(e.X, e.Y)))
            {
                MessageBox.Show("배터리가 충전되었습니다.");
            }
        }

        #endregion
        #region 폼 마우스 UP 처리하기 - Form_MouseUp(sender, e)

        /// <summary>
        /// 폼 마우스 UP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_MouseUp(object sender, MouseEventArgs e)
        {
            if(this.mouseDownInButton && e.Button == MouseButtons.Left)
            {
                DrawButton(this.buttonRectangle, null, true);

                this.mouseDownInButton = false;

                if(this.buttonRectangle.Contains(e.X, e.Y))
                {
                    MessageBox.Show("버튼을 눌렀습니다.");
                }
            }
        }

        #endregion
        #region 폼 페인트시 처리하기 - Form_Paint(sender, e)

        /// <summary>
        /// 폼 페인트시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Paint(object sender, PaintEventArgs e)
        {
            Graphics graphics = e.Graphics;

            DrawButton(this.buttonRectangle, graphics, true);

            graphics.FillRectangle(Brushes.Red, this.redRectangle);
        }

        #endregion
        #region 캡처 변경시 윈도우 캡처 변경시 처리하기 - captureChangedWindow_CaptureChanged()

        /// <summary>
        /// 캡처 변경시 윈도우 캡처 변경시 처리하기
        /// </summary>
        private void captureChangedWindow_CaptureChanged()
        {
            this.mouseDownInButton = false;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 버튼 그리기 - DrawButton(rectangle, graphics, buttonUp)

        /// <summary>
        /// 버튼 그리기
        /// </summary>
        /// <param name="rectangle">사각형</param>
        /// <param name="graphics">그래픽스</param>
        /// <param name="buttonUp">버튼 UP 여부</param>
        private void DrawButton(Rectangle rectangle, Graphics graphics, bool buttonUp)
        {
            bool disposeGraphics = false;

            if(graphics == null)
            {
                graphics = CreateGraphics();

                disposeGraphics = true;
            }

            GraphicsHelper helper = new GraphicsHelper(graphics);

            helper.DrawRectangle(Pens.Black, rectangle);

            Rectangle innerRectangle = new Rectangle(rectangle.Location, rectangle.Size);

            innerRectangle.Inflate(new Size(-1, -1));

            if(buttonUp)
            {
                helper.Draw3DRectangle(innerRectangle, Style3D.Raised, 2);
            }
            else
            {
                helper.FillRectangle(Brushes.LightGray, innerRectangle);
            }

            if(disposeGraphics)
            {
                graphics.Dispose();
            }
        }

        #endregion
    }
}